
/*
 * Copyright (c) 2016 XCSP3 Team (contact@xcsp.org)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

import static org.junit.Assert.assertEquals;

import java.io.ByteArrayInputStream;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.stream.Collectors;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;
import org.xcsp.checker.SolutionChecker;

/**
 * @author Gilles Audemard and Christophe Lecoutre
 */
@RunWith(Parameterized.class)
public class TestSolutions {
	static String path = System.getProperty("user.dir") + "/instancesTest/";

	@Parameters(name = "{index}: {0} has {1} solutions")
	public static Collection<Object[]> data() {
		System.out.println(System.getProperty("user.dir") + "\n" + (System.getProperty("user.dir") + path));
		// not tested : Steiner, Rcpsp, Supersolutions, strippacking, schurrlemma, rlfap

		Map<Object, Object> map = new LinkedHashMap<>();
		map.put("AllInterval-005.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] y[] </list>  <values> 4 0 3 1 2 4 3 2 1 </values>  </instantiation>");
		map.put("Allergy.xml",
				"<instantiation id='sol1' type='solution'>  <list> eggs mold nuts ragweed baxter lemmon malone vanfleet </list>  <values> Rick Hugh Janet Debra Rick Debra Hugh Janet </values>  </instantiation>");
		map.put("Zebra.xml",
				"<instantiation id='sol1' type='solution'>  <list> yellow green red white blue italy spain japan england norway painter sculptor diplomat pianist doctor cat zebra bear snails horse milk water tea coffee juice </list>  <values> 1 4 5 3 2 4 2 1 5 3 2 4 1 5 3 1 3 5 4 2 3 2 4 1 5 </values>  </instantiation>");
		map.put("Bibd-sc-06-050-25-03-10.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] s[][][] </list>  <values> 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 0 0 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 0 0 0 0 0 0 0 1 1 1 1 1 0 1 1 1 1 1 1 1 0 0 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 1 1 1 0 0 0 1 1 1 1 0 0 0 1 1 1 0 0 0 0 0 1 1 1 1 0 0 1 1 1 1 0 0 0 1 1 1 1 1 1 1 1 1 0 1 1 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 0 0 1 1 1 1 0 0 0 0 1 1 1 0 0 0 0 1 1 1 1 1 1 0 0 1 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 1 1 1 0 0 0 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 0 1 1 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 0 0 1 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 1 1 1 1 0 0 0 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 0 0 0 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 1 1 0 0 0 0 0 0 1 1 0 0 0 0 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 1 1 1 1 1 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 1 1 1 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * </values>  </instantiation>");
		map.put("Bibd-sum-06-050-25-03-10.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 0 0 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 0 0 0 0 0 0 0 1 1 1 1 1 0 1 1 1 1 1 1 1 0 0 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 1 1 1 0 0 0 1 1 1 1 0 0 0 1 1 1 0 0 0 1 1 1 1 1 1 0 0 0 1 1 1 0 0 0 0 1 1 1 1 1 1 1 1 0 0 1 0 0 1 1 0 0 0 0 1 1 0 0 0 0 0 0 0 1 1 1 1 1 1 0 0 0 0 1 1 1 1 1 0 0 0 1 1 1 1 0 0 0 1 1 1 1 1 1 1 0 1 1 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 </values>  </instantiation>");
		map.put("BinPacking-tab-n1c1w4a.xml",
				"<instantiation id='sol1' type='solution' cost='5'>  <list> x b[][] </list>  <values> 5 99 0 0 95 0 0 93 0 0 92 0 0 91 0 0 89 0 0 89 0 0 88 0 0 88 0 0 85 0 0 84 0 0 84 0 0 84 0 0 80 0 0 80 0 0 79 0 0 77 0 0 76 0 0 72 0 0 69 0 0 65 31 0 64 32 0 64 31 0 63 34 0 63 30 0 60 35 0 56 40 0 56 35 0 53 41 0 53 40 0 52 40 0 51 49 0 50 50 0 49 41 0 47 44 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 </values>  </instantiation>");
		map.put("BinPacking-mdd-n1c1w4a.xml",
				"<instantiation id='sol1' type='solution' cost='5'>  <list> x b[][] </list>  <values> 5 99 0 0 95 0 0 93 0 0 92 0 0 91 0 0 89 0 0 89 0 0 88 0 0 88 0 0 85 0 0 84 0 0 84 0 0 84 0 0 80 0 0 80 0 0 79 0 0 77 0 0 76 0 0 72 0 0 69 0 0 65 31 0 64 32 0 64 31 0 63 34 0 63 30 0 60 35 0 56 40 0 56 35 0 53 41 0 53 40 0 52 40 0 51 49 0 50 50 0 49 41 0 47 44 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 </values>  </instantiation>");
		map.put("BinPacking-sum-n1c1w4a.xml",
				"<instantiation id='sol2' type='solution' cost='5'>  <list> x b[][] </list>  <values> 5 99 0 0 95 0 0 93 0 0 92 0 0 91 0 0 89 0 0 89 0 0 88 0 0 88 0 0 85 0 0 84 0 0 84 0 0 84 0 0 80 0 0 80 0 0 79 0 0 77 0 0 76 0 0 72 0 0 69 0 0 65 34 0 64 30 0 64 0 0 63 35 0 63 35 0 60 40 0 56 40 0 56 40 0 53 41 0 53 41 0 52 47 0 51 44 0 50 49 0 50 49 0 32 31 31 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 </values>  </instantiation>");
		map.put("Blackhole-04-3-00.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] y[] </list>  <values> 0 15 14 11 10 3 2 7 12 1 8 5 4 9 6 13 0 9 6 5 12 11 14 7 10 13 4 3 8 15 2 1 </values>  </instantiation>");
		map.put("BusScheduling-cnt-t1.xml",
				"<instantiation id='sol2' type='solution' cost='7'>  <list> x[] </list>  <values> 0 1 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 </values>  </instantiation>");
		map.put("CarSequencing-dingbas.xml",
				"<instantiation id='sol1' type='solution'>  <list> cls[] opt[][] </list>  <values> 0 1 5 2 4 3 3 4 2 5 1 0 1 1 0 0 0 0 1 0 1 1 0 0 0 0 1 0 0 1 1 0 1 0 0 0 1 0 1 0 0 1 0 1 0 1 0 1 0 0 0 1 0 0 1 1 1 0 0 0 </values>  </instantiation>");
		map.put("ChessboardColoration-07-07.xml",
				"<instantiation id='sol2' type='solution' cost='2'>  <list> x[][] </list>  <values> 1 0 1 0 2 2 0 0 1 1 1 0 2 0 0 2 1 0 1 1 1 0 0 0 1 2 0 1 1 1 0 0 0 1 2 0 1 2 2 1 2 2 2 1 2 0 2 0 1 </values>  </instantiation>");
		map.put("ColouredQueens-07.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 1 2 4 3 6 5 0 3 6 5 0 1 2 4 0 1 2 4 3 6 5 4 3 6 5 0 1 2 5 0 1 2 4 3 6 2 4 3 6 5 0 1 6 5 0 1 2 4 3 </values>  </instantiation>");
		map.put("CostasArray-12.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] y[] </list>  <values> 11 1 6 3 7 9 10 5 8 4 2 12 10 -5 3 -4 -2 -1 5 -3 4 2 -10 5 -2 -1 -6 -3 4 2 1 6 -8 8 -6 -3 -7 2 1 6 3 -4 4 -8 -4 -2 -1 5 8 -7 2 -9 1 -5 3 7 -2 1 -4 -2 -1 5 -3 6 -7 2 1 -5 3 -3 4 -9 7 -1 -6 </values>  </instantiation>");
		map.put("Crossword-lex-vg-5-6.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 0 6 7 0 18 19 20 17 0 13 8 0 17 14 3 4 13 19 0 21 4 13 6 4 11 4 18 19 4 17 </values>  </instantiation>");
		map.put("CryptoPuzzle-cross-roads-danger.xml",
				"<instantiation id='sol1' type='solution'>  <list> l[] C[] </list>  <values> 5 * 9 1 4 * 7 * * * * * * 8 2 * * 6 3 * * * * * * * 0 0 0 0 0 1 </values>  </instantiation>");
		map.put("Cutstock-small.xml",
				"<instantiation id='sol1' type='solution' cost='4'>  <list> p[] r[][] </list>  <values> 1 1 1 1 0 0 0 1 0 1 1 0 0 0 2 0 0 0 3 0 0 0 0 0 0 0 0 0 </values>  </instantiation>");
		map.put("DistinctVectors-30-050-02.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 </values>  </instantiation>");
		map.put("Domino-300-300.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 299 </values>  </instantiation>");
		map.put("driverlogw-09.xml",
				"<instantiation id='sol1' type='solution'>  <list> x0[] x1[] x2[] x3[] x4[] x5[] x6[] x7[] x8[] </list>  <values> 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 2 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 3 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 2 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 5 0 0 0 0 1 0 0 0 2 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 3 1 0 1 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 1 0 0 0 0 1 0 0 0 1 1 0 1 1 1 0 0 0 0 4 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 1 0 1 1 0 0 0 0 0 0 0 0 0 0 0 5 0 1 6 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 1 0 0 0 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 0 0 8 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 1 0 0 2 0 0 0 0 0 1 0 0 0 1 0 0 0 0 1 0 0 0 1 1 0 1 1 0 0 0 0 0 0 0 0 2 1 0 1 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 1 0 0 9 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 1 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 1 1 1 0 1 0 1 1 1 1 0 0 1 0 1 1 1 1 1 1 1 1 1 0 1 0 0 1 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 0 0 </values>  </instantiation>");
		map.put("Fastfood-ff10.xml",
				"<instantiation id='sol9' type='solution' cost='704'>  <list> x[] drd[][] dr[] </list>  <values> 41 116 198 237 352 229 154 72 33 82 229 154 72 33 82 59 16 98 137 252 60 15 97 136 251 428 353 271 232 117 1 76 158 197 312 1 76 158 197 312 196 121 39 0 115 196 121 39 0 115 9 66 148 187 302 9 66 148 187 302 311 236 154 115 0 311 236 154 115 0 64 11 93 132 247 64 11 93 132 247 90 15 67 106 221 272 197 115 76 39 272 197 115 76 39 18 93 175 214 329 18 93 175 214 329 0 75 157 196 311 0 75 157 196 311 88 13 69 108 223 165 90 8 31 146 164 89 7 32 147 184 109 27 12 127 184 109 27 12 127 83 8 74 113 228 296 221 139 100 15 296 221 139 100 15 20 55 137 176 291 134 59 23 62 177 356 281 199 160 45 356 281 199 160 45 13 62 144 183 298 222 147 65 26 89 75 0 82 121 236 75 0 82 121 236 332 257 175 136 21 157 82 0 39 154 157 82 0 39 154 139 64 18 57 172 139 64 18 57 172 33 33 16 15 117 1 1 0 0 9 9 0 0 11 11 15 39 39 18 18 0 0 13 8 7 12 12 8 15 15 20 23 45 45 13 26 0 0 21 0 0 18 18 </values>  </instantiation>");
		map.put("GolombRuler-09-a3.xml",
				"<instantiation id='sol2' type='solution' cost='61'>  <list> x[] y[][] </list>  <values> 0 1 3 7 12 20 30 45 61 * 1 3 7 12 20 30 45 61 * * 2 6 11 19 29 44 60 * * * 4 9 17 27 42 58 * * * * 5 13 23 38 54 * * * * * 8 18 33 49 * * * * * * 10 25 41 * * * * * * * 15 31 * * * * * * * * 16 * * * * * * * * * </values>  </instantiation>");
		map.put("GolombRuler-09-a4.xml",
				"<instantiation id='sol7' type='solution' cost='44'>  <list> x[] </list>  <values> 0 1 5 12 25 27 35 41 44 </values>  </instantiation>");
		map.put("GracefulGraph-K02-P04.xml",
				"<instantiation id='sol1' type='solution'>  <list> cn[][] ce[][][] cie[][] </list>  <values> 4 3 2 9 5 1 0 10 * 1 * * * 7 * * * 4 * * * 10 * * 2 6 3 8 5 9 </values>  </instantiation>");
		map.put("GraphColoring-3-fullins-4.xml",
				"<instantiation id='sol1' type='solution' cost='6'>  <list> x[] </list>  <values> 3 4 2 2 3 3 2 2 4 4 2 4 5 6 3 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 1 2 3 4 5 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 1 2 3 4 </values>  </instantiation>");
		map.put("GraphColoring-qwhdec-o5-h10-1.xml",
				"<instantiation id='sol1' type='solution' cost='4'>  <list> x[] </list>  <values> 0 1 4 3 2 3 4 1 2 0 1 0 2 4 3 2 3 0 1 4 4 2 3 0 1 </values>  </instantiation>");
		map.put("Hanoi-05.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 </values>  </instantiation>");
		map.put("Kakuro-easy-000-ext.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> * * * * * * * * 5 8 1 * * 8 6 9 4 * * 9 8 * 3 1 * * 7 9 2 3 * * 9 8 6 * </values>  </instantiation>");
		map.put("Kakuro-easy-000-sumdiff.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> * * * * * * * * 5 8 1 * * 8 6 9 4 * * 9 8 * 3 1 * * 7 9 2 3 * * 9 8 6 * </values>  </instantiation>");
		map.put("Knapsack-30-100-00.xml",
				"<instantiation id='sol3' type='solution' cost='709'>  <list> x[] </list>  <values> 1 0 1 0 0 1 0 0 0 1 1 1 0 0 1 1 1 1 0 0 0 1 0 0 1 0 0 0 0 0 </values>  </instantiation>");
		map.put("KnightTour-06-ext03.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 0 8 19 30 26 34 23 27 35 22 9 5 16 29 33 25 12 1 14 6 2 10 21 17 4 15 11 3 7 18 31 20 28 32 24 13 </values>  </instantiation>");
		map.put("KnightTour-06-int.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 0 8 4 15 2 6 14 1 9 5 16 29 33 25 12 20 31 18 7 3 11 22 35 27 19 30 26 34 23 10 21 17 28 32 24 13 </values>  </instantiation>");
		map.put("Langford-3-10.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 0 3 18 5 13 14 8 11 7 1 2 6 22 10 19 21 16 20 17 12 4 9 26 15 25 28 24 29 27 23 </values>  </instantiation>");
		map.put("LangfordBin-08.xml",
				"<instantiation id='sol1' type='solution'>  <list> v[] p[] </list>  <values> 3 6 8 1 3 1 7 4 6 5 2 8 4 2 7 5 5 3 13 10 4 0 12 7 15 9 8 1 14 6 11 2 </values>  </instantiation>");
		map.put("qwh-o30-h374-01.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 19 18 17 28 2 14 16 24 11 3 6 20 5 13 0 23 21 27 26 12 4 22 9 25 10 15 8 1 29 7 26 17 7 4 28 21 27 16 19 6 29 5 20 2 23 1 25 0 12 10 22 8 14 13 18 11 15 9 3 24 14 0 29 7 3 5 18 2 10 13 15 27 4 20 9 26 1 25 11 19 24 17 21 23 12 6 22 8 28 16 23 3 19 24 8 16 5 9 15 20 22 17 11 7 12 27 26 10 25 21 28 29 4 0 1 18 14 6 2 13 8 15 3 2 0 24 1 14 16 10 21 7 9 17 19 13 23 5 20 28 29 6 27 26 11 25 12 4 18 22 21 6 0 9 16 11 10 23 14 15 3 19 13 12 8 24 22 17 27 4 18 1 26 29 2 5 20 25 7 28 13 11 5 22 9 28 21 3 26 1 12 23 8 15 24 2 16 29 7 18 6 20 0 14 19 17 27 10 25 4 11 20 2 6 12 0 8 29 27 24 1 13 19 22 18 21 9 14 16 25 26 28 15 10 3 4 7 17 5 23 0 19 27 14 10 18 28 6 5 9 7 26 2 16 17 20 4 13 22 23 11 12 24 8 25 21 3 29 15 1 1 2 20 23 25 7 4 0 13 11 28 16 12 5 29 22 17 9 24 27 3 14 8 15 26 10 21 18 6 19 15 27 9 3 18 10 17 8 23 14 5 24 6 28 22 29 19 20 1 2 0 21 13 4 16 12 26 7 11 25 2 14 12 1 4 17 19 28 25 26 8 6 21 0 27 3 18 15 13 9 5 16 10 20 7 23 29 22 24 11 3 25 11 20 5 22 2 12 6 16 24 29 15 4 28 8 14 18 19 26 21 23 7 9 27 0 1 13 17 10 10 26 15 0 27 20 11 18 2 25 13 12 7 21 1 9 24 4 14 6 19 3 23 28 29 8 16 5 22 17 17 1 28 10 7 8 9 27 29 12 11 21 22 3 2 0 13 23 4 24 15 19 6 18 5 14 25 20 16 26 4 23 16 5 24 6 25 22 9 18 0 3 28 1 7 17 29 19 21 11 8 27 2 12 13 20 10 26 14 15 29 7 4 17 6 9 15 13 20 2 19 1 10 8 25 28 11 12 0 5 14 18 16 3 24 22 23 21 26 27 22 29 8 26 20 15 6 19 24 27 4 9 23 10 3 16 12 7 5 14 25 2 18 17 28 1 13 11 0 21 24 21 18 27 26 1 14 20 4 5 25 28 17 19 13 12 15 2 10 0 9 11 3 22 8 7 6 16 23 29 5 4 25 11 1 23 22 17 7 28 2 14 24 18 10 19 20 21 15 8 27 26 29 16 6 13 9 0 12 3 27 13 21 29 17 2 12 26 1 7 18 15 16 9 14 6 8 28 3 22 10 4 25 5 23 19 11 24 20 0 16 10 22 18 23 26 29 7 12 0 27 8 3 25 4 5 28 11 2 13 1 15 17 21 20 9 24 14 19 6 12 22 14 16 15 27 20 25 0 19 10 4 1 6 26 7 5 24 8 3 17 13 11 2 9 29 28 23 21 18 18 9 10 12 13 29 26 4 3 21 23 22 25 24 11 14 0 1 6 20 7 5 28 19 15 16 17 27 8 2 28 24 26 19 11 4 7 10 17 8 9 2 29 23 21 25 6 22 18 16 13 0 20 1 14 3 5 15 27 12 20 8 1 25 14 19 24 5 21 22 16 18 0 26 15 11 7 3 17 29 23 10 12 6 4 27 2 28 13 9 9 28 24 13 19 25 3 11 8 4 26 0 14 29 5 15 10 6 23 17 16 7 22 27 21 2 18 12 1 20 25 5 6 8 21 13 23 15 18 29 17 10 27 11 20 4 2 16 28 1 12 24 19 7 22 26 0 3 9 14 7 16 13 21 22 12 0 1 28 23 14 25 18 27 6 10 3 26 29 15 20 9 5 11 17 24 19 2 4 8 6 12 23 15 29 3 13 21 22 17 20 11 26 14 16 18 27 8 9 7 2 25 1 24 0 28 4 19 10 5 </values>  </instantiation>");
		map.put("qcp-15-120-00_X2.xml",
				"<instantiation id='sol1' type='solution'>  <list> x0 x1 x2 x3 x4 x5 x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18 x19 x20 x21 x22 x23 x24 x25 x26 x27 x28 x29 x30 x31 x32 x33 x34 x35 x36 x37 x38 x39 x40 x41 x42 x43 x44 x45 x46 x47 x48 x49 x50 x51 x52 x53 x54 x55 x56 x57 x58 x59 x60 x61 x62 x63 x64 x65 x66 x67 x68 x69 x70 x71 x72 x73 x74 x75 x76 x77 x78 x79 x80 x81 x82 x83 x84 x85 x86 x87 x88 x89 x90 x91 x92 x93 x94 x95 x96 x97 x98 x99 x100 x101 x102 x103 x104 x105 x106 x107 x108 x109 x110 x111 x112 x113 x114 x115 x116 x117 x118 x119 x120 x121 x122 x123 x124 x125 x126 x127 x128 x129 x130 x131 x132 x133 x134 x135 x136 x137 x138 x139 x140 x141 x142 x143 x144 x145 x146 x147 x148 x149 x150 x151 x152 x153 x154 x155 x156 x157 x158 x159 x160 x161 x162 x163 x164 x165 x166 x167 x168 x169 x170 x171 x172 x173 x174 x175 x176 x177 x178 x179 x180 x181 x182 x183 x184 x185 x186 x187 x188 x189 x190 x191 x192 x193 x194 x195 x196 x197 x198 x199 x200 x201 x202 x203 x204 x205 x206 x207 x208 x209 x210 x211 x212 x213 x214 x215 x216 x217 x218 x219 x220 x221 x222 x223 x224 </list>  <values> 5 11 8 7 0 12 6 14 10 3 9 13 1 4 2 8 6 10 14 13 11 1 3 12 2 0 7 4 5 9 6 1 5 0 12 8 13 7 3 9 14 10 11 2 4 1 5 2 6 11 14 7 13 4 10 12 3 8 9 0 2 13 9 4 14 6 11 8 1 7 10 5 3 0 12 9 2 14 5 7 1 3 4 8 12 11 6 0 13 10 10 4 11 2 3 5 14 0 7 13 8 9 12 1 6 4 12 13 9 10 2 0 1 11 14 6 8 5 7 3 13 14 7 11 1 4 2 12 6 8 3 0 9 10 5 3 10 4 1 9 0 5 6 14 11 7 2 13 12 8 14 0 12 3 8 13 9 5 2 4 1 11 10 6 7 11 9 3 8 2 7 4 10 0 1 5 12 6 14 13 12 3 6 13 4 9 10 11 5 0 2 1 7 8 14 7 8 0 10 6 3 12 2 9 5 13 4 14 11 1 0 7 1 12 5 10 8 9 13 6 4 14 2 3 11 </values>  </instantiation>");
		map.put("LowAutocorrelation-015.xml",
				"<instantiation id='sol6' type='solution' cost='15'>  <list> x[] y[][] c[] s[] </list>  <values> -1 1 -1 -1 1 -1 -1 -1 1 -1 -1 -1 1 1 1 -1 -1 1 -1 -1 1 1 -1 -1 1 1 -1 1 1 1 -1 -1 1 -1 1 -1 1 -1 1 -1 -1 1 * 1 1 1 1 -1 -1 1 1 -1 -1 -1 -1 * * -1 -1 1 1 1 1 1 1 1 -1 -1 * * * 1 -1 1 -1 -1 1 1 -1 1 -1 * * * * 1 -1 -1 1 -1 1 -1 -1 1 * * * * * 1 1 1 1 -1 -1 -1 -1 * * * * * * -1 -1 1 1 1 -1 -1 * * * * * * * 1 -1 1 -1 1 -1 * * * * * * * * 1 -1 -1 -1 1 * * * * * * * * * 1 1 -1 -1 * * * * * * * * * * -1 1 -1 * * * * * * * * * * * -1 1 * * * * * * * * * * * * -1 * * * * * * * * * * * * * 0 -1 0 3 0 -1 0 -1 0 -1 0 -1 0 -1 0 1 0 9 0 1 0 1 0 1 0 1 0 1 </values>  </instantiation>");
		map.put("MagicSequence-008-ca.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 4 2 1 0 1 0 0 0 </values>  </instantiation>");
		map.put("MagicSequence-008-co.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 4 2 1 0 1 0 0 0 </values>  </instantiation>");
		map.put("MagicSquare-9-f10-01.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 71 3 7 73 9 5 55 72 74 15 63 51 21 69 45 28 13 64 76 8 31 77 4 75 78 18 2 47 66 40 12 14 53 52 36 49 35 60 43 23 67 37 27 44 33 16 65 38 20 70 32 25 42 61 17 29 34 41 58 59 24 57 50 81 19 79 80 10 1 54 39 6 11 56 46 22 68 62 26 48 30 </values>  </instantiation>");
		map.put("MagicSquare-4-table.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 1 6 11 16 12 15 2 5 8 3 14 9 13 10 7 4 </values>  </instantiation>");
		map.put("MagicSquare-6-sum.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 27 36 6 4 3 35 23 11 31 32 13 1 16 20 17 21 28 9 26 24 25 7 19 10 14 12 30 18 15 22 5 8 2 29 33 34 </values>  </instantiation>");
		map.put("MagicSquare-6-mdd.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 9 28 27 15 26 6 25 32 16 10 21 7 5 3 2 30 35 36 19 13 14 33 24 8 31 18 29 12 1 20 22 17 23 11 4 34 </values>  </instantiation>");
		map.put("Mario-easy-4.xml",
				"<instantiation id='sol5' type='solution' cost='545'>  <list> s[] f[] g[] </list>  <values> 4 0 7 8 12 14 13 6 9 2 1 11 5 10 3 111 0 136 340 281 7 180 21 115 302 4 0 255 18 189 0 0 3 58 67 5 11 46 62 27 92 0 52 60 62 </values>  </instantiation>");
		map.put("MarketSplit-01.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 1 0 0 0 1 1 1 0 0 1 1 0 0 0 0 1 0 0 1 1 0 1 0 1 0 0 0 1 1 1 </values>  </instantiation>");
		map.put("MultiKnapsack-1-01.xml", "<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 0 1 1 0 0 1 </values>  </instantiation>");
		map.put("MultiKnapsack-1-0_X2.xml", "<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 0 1 1 0 0 1 </values>  </instantiation>");
		map.put("Nonogram-001-regular.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 1 1 1 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 1 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 1 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 1 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 1 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 1 1 0 1 1 1 1 0 0 0 0 0 1 1 1 1 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 0 </values>  </instantiation>");
		map.put("Nonogram-001-table.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 1 1 1 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 1 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 1 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 1 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 1 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 1 1 0 1 1 1 1 0 0 0 0 0 1 1 1 1 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 0 </values>  </instantiation>");
		map.put("Opd-07-007-003.xml",
				"<instantiation id='sol3' type='solution' cost='1'>  <list> z x[][] </list>  <values> 1 0 0 0 0 1 1 1 0 0 1 1 0 0 1 0 1 0 1 0 1 0 0 1 1 0 1 0 0 1 0 0 1 1 0 0 1 0 1 0 0 1 0 1 1 0 0 0 0 1 </values>  </instantiation>");
		map.put("Ortholatin-005.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] y[][] z[] </list>  <values> 0 1 2 3 4 3 2 4 1 0 4 3 1 0 2 2 0 3 4 1 1 4 0 2 3 0 1 2 3 4 1 4 0 2 3 2 0 3 4 1 3 2 4 1 0 4 3 1 0 2 0 6 12 18 24 16 14 20 7 3 22 15 8 4 11 13 2 19 21 5 9 23 1 10 17 </values>  </instantiation>");
		map.put("Primes-15-20-2-1.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> * 3 * * 23 31 16 41 29 3 * * * 11 * * * 13 * 3 * 2 * * 2 13 * 43 19 31 47 41 * * * * 43 3 * * 29 28 17 13 * * 23 * * * * * * 3 * * 5 * 3 * * * * * * * * * 43 * * 2 * * * * * * * 13 * * * * 23 * * * 47 * * * 37 * * * 3 17 * 47 </values>  </instantiation>");
		map.put("PrizeCollecting-15-3-5-0.xml",
				"<instantiation id='sol4' type='solution' cost='20'>  <list> s[] p[] g[] </list>  <values> 1 2 3 4 9 0 5 6 -1 14 -1 -1 7 12 13 0 1 2 3 4 11 10 9 -1 5 -1 -1 8 7 6 -3 1 3 1 3 -5 -1 7 0 -2 0 0 6 6 4 </values>  </instantiation>");
		map.put("Pb-robin08.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 </values>  </instantiation>");
		map.put("Pb-gr-05.xml",
				"<instantiation id='sol2' type='solution' cost='11'>  <list> x[] </list>  <values> 1 1 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 0 0 1 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 1 1 0 0 1 1 1 1 1 1 1 1 1 1 0 1 0 0 1 1 0 0 0 0 </values>  </instantiation>");
		map.put("QuadraticAssignment-bur26a.xml",
				"<instantiation id='sol1' type='solution' cost='3042272'>  <list> x[] d[][] </list>  <values> 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 * 348 316 74 12 181 338 309 35 3 84 714 367 1153 7 71 0 687 432 507 975 38 6 8 7 15 * * 0 4 1300 12 41 18 183 6 3 102 2 7 84 0 0 150 84 54 148 2 13 0 1 11 * * * 9 12 0 1 3100 3 1 209 9 3 1 22 0 0 4 2 3 1 0 2 0 0 0 * * * * 2729 10 10 6 1186 0 4 48 46 30 103 11 0 102 36 34 160 2 14 0 3 1 * * * * * 142 391 398 2329 4 132 747 479 4754 32 51 3 4501 1311 512 326 26 111 43 6 68 * * * * * * 31 4 91 2 3 76 2 16 104 5 1 163 30 257 45 2 1 0 1 16 * * * * * * * 2 138 3 33 126 7 41 39 0 0 198 199 149 100 2 1 0 5 10 * * * * * * * * 207 2 26 230 125 204 132 1 0 596 113 596 107 5 67 0 4 12 * * * * * * * * * 2 151 370 292 2100 220 18 3 250 1038 1008 25 65 2 4 0 42 * * * * * * * * * * 0 0 0 1 12 0 0 0 0 1 28 0 0 0 0 0 * * * * * * * * * * * 118 1 11 247 4 0 113 32 218 116 1 4 0 1 3 * * * * * * * * * * * * 27 43 126 11 0 9 239 332 186 12 1 0 28 21 * * * * * * * * * * * * * 4 101 80 0 1 44 86 101 0 6 0 0 0 * * * * * * * * * * * * * * 184 13 0 21 496 701 194 24 36 1 5 159 * * * * * * * * * * * * * * * 73 0 590 157 114 26 17 48 3 7 122 * * * * * * * * * * * * * * * * 0 391 13 42 44 0 0 0 0 0 * * * * * * * * * * * * * * * * * 1 0 0 16 0 0 0 0 0 * * * * * * * * * * * * * * * * * * 443 601 314 20 77 2 3 94 * * * * * * * * * * * * * * * * * * * 1432 92 42 46 0 19 18 * * * * * * * * * * * * * * * * * * * * 197 8 98 0 8 248 * * * * * * * * * * * * * * * * * * * * * 13 9 4 0 12 * * * * * * * * * * * * * * * * * * * * * * 4 0 0 0 * * * * * * * * * * * * * * * * * * * * * * * 0 0 0 * * * * * * * * * * * * * * * * * * * * * * * * 1 1 * * * * * * * * * * * * * * * * * * * * * * * * * 0 * * * * * * * * * * * * * * * * * * * * * * * * * * </values>  </instantiation>");
		map.put("QuasiGroup-3-04.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] y[][] </list>  <values> 0 2 3 1 3 1 0 2 1 3 2 0 2 0 1 3 * 11 13 6 14 * 3 8 7 12 * 1 9 2 4 * </values>  </instantiation>");
		map.put("QuasiGroup-7-09.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] y[][] </list>  <values> 0 2 3 8 6 7 5 1 4 8 1 4 7 3 0 2 5 6 4 7 2 6 5 3 8 0 1 6 0 1 3 2 4 7 8 5 7 5 6 0 4 8 1 2 3 2 6 8 1 0 5 4 3 7 1 3 7 5 8 2 6 4 0 3 8 5 4 1 6 0 7 2 5 4 0 2 7 1 3 6 8 * 4 6 5 1 3 2 8 7 4 * 5 8 0 2 7 6 3 6 5 * 7 8 1 0 3 4 5 8 7 * 6 0 4 2 1 1 0 8 6 * 7 3 5 2 3 2 1 0 7 * 8 4 6 2 7 0 4 3 8 * 1 5 8 6 3 2 5 4 1 * 0 7 3 4 1 2 6 5 0 * </values>  </instantiation>");
		map.put("QueenAttacking-06.xml",
				"<instantiation id='sol4' type='solution' cost='0'>  <list> q x[] b[] </list>  <values> 9 0 8 4 15 2 6 14 1 12 25 33 29 16 5 9 17 21 10 23 34 26 13 24 32 28 20 31 18 7 3 11 22 35 27 19 30 0 0 0 0 0 0 0 0 0 0 0 </values>  </instantiation>");
		map.put("Queens-0008-m1.xml", "<instantiation id='sol1' type='solution'>  <list> q[] </list>  <values> 0 6 4 7 1 3 5 2 </values>  </instantiation>");
		map.put("RadarSurveillance-8-24-3-2-00.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> * 0 0 0 * * * 2 0 0 0 * 1 0 0 0 3 2 1 3 0 0 1 0 1 3 0 0 0 2 2 0 0 1 0 3 3 3 2 3 * * 0 0 0 0 0 3 3 2 0 0 0 2 1 0 0 0 * 1 0 0 0 0 0 0 2 1 2 0 0 3 3 0 0 0 3 3 3 3 3 0 1 1 0 0 0 2 0 0 3 * 1 1 1 2 0 0 1 1 * 1 3 0 0 0 0 0 0 * * 0 1 0 0 3 * * * * 3 3 * * 1 1 3 0 * * 3 3 2 0 * * 1 2 3 * * * 3 0 </values>  </instantiation>");
		map.put("Ramsey-12.xml",
				"<instantiation id='sol3' type='solution' cost='2'>  <list> x[][] </list>  <values> * 1 1 2 0 2 1 2 0 0 0 0 * * 2 1 1 2 0 0 0 0 1 2 * * * 1 0 1 0 2 0 0 1 0 * * * * 0 0 0 1 1 0 2 1 * * * * * 1 2 0 1 2 2 1 * * * * * * 2 1 0 1 0 0 * * * * * * * 1 2 1 0 1 * * * * * * * * 2 2 0 0 * * * * * * * * * 1 2 2 * * * * * * * * * * 1 2 * * * * * * * * * * * 1 * * * * * * * * * * * * </values>  </instantiation>");
		map.put("RoomMate-sr0050-int.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 9 4 2 8 4 2 5 0 0 6 6 3 2 3 0 9 2 5 5 20 1 23 4 0 2 3 6 1 6 2 3 1 2 14 22 1 3 2 4 0 1 3 5 0 7 2 0 1 0 3 </values>  </instantiation>");
		map.put("Sat-flat200-00-clause.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 0 1 0 0 0 1 0 1 0 0 0 1 0 0 1 0 0 1 0 1 0 0 0 1 0 1 0 1 0 0 1 0 0 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 1 0 0 0 0 1 0 0 1 0 0 1 0 0 1 0 0 1 0 0 1 1 0 0 0 0 1 0 0 1 0 0 1 0 0 1 1 0 0 0 0 1 1 0 0 1 0 0 0 1 0 1 0 0 1 0 0 0 0 1 0 1 0 0 0 1 1 0 0 1 0 0 0 1 0 0 1 0 1 0 0 0 0 1 0 0 1 0 0 1 0 1 0 0 1 0 0 1 0 0 1 0 0 0 1 0 1 0 0 0 1 0 0 1 0 1 0 0 1 0 0 1 0 1 0 0 1 0 0 0 0 1 0 1 0 1 0 0 1 0 0 0 1 0 1 0 0 0 1 0 0 1 0 0 1 0 0 1 0 0 0 1 0 0 1 1 0 0 1 0 0 0 1 0 1 0 0 0 0 1 1 0 0 0 1 0 0 0 1 0 0 1 0 1 0 0 1 0 0 0 1 1 0 0 0 1 0 0 1 0 1 0 0 0 0 1 0 1 0 1 0 0 0 1 0 0 1 0 1 0 0 0 0 1 0 0 1 0 1 0 0 1 0 0 0 1 0 0 1 1 0 0 1 0 0 0 1 0 0 0 1 1 0 0 0 1 0 1 0 0 0 1 0 0 0 1 0 1 0 0 1 0 0 0 1 0 1 0 0 1 0 0 0 1 1 0 0 1 0 0 1 0 0 1 0 0 0 0 1 0 0 1 1 0 0 0 1 0 1 0 0 0 0 1 0 0 1 0 1 0 1 0 0 1 0 0 0 1 0 1 0 0 0 1 0 0 0 1 1 0 0 0 0 1 1 0 0 1 0 0 1 0 0 0 0 1 1 0 0 0 1 0 0 1 0 0 0 1 0 1 0 0 0 1 0 1 0 0 0 1 1 0 0 0 0 1 0 0 1 0 0 1 1 0 0 0 0 1 0 1 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 1 0 0 1 0 0 1 0 0 1 0 1 0 1 0 0 1 0 0 0 1 0 1 0 0 0 1 0 0 1 0 1 0 0 0 0 1 1 0 0 0 0 1 0 1 0 1 0 0 1 0 0 0 0 1 0 0 1 0 0 1 0 0 1 0 0 1 1 0 0 1 0 0 0 0 1 0 0 1 0 0 1 0 0 1 0 1 0 0 0 1 1 0 0 0 1 0 0 1 0 0 0 1 0 1 0 0 0 1 0 0 1 0 0 1 0 0 1 1 0 0 0 0 1 0 1 0 </values>  </instantiation>");
		map.put("Taillard-os-04-04-0.xml",
				"<instantiation id='sol3' type='solution' cost='193'>  <list> s[][] e[] d[][] w[][] sd[][] </list>  <values> 0 34 89 116 0 89 167 182 0 29 129 167 0 34 129 159 177 191 186 193 34 54 2 61 89 70 15 9 28 87 38 19 29 95 7 34 0 2 1 3 1 2 0 3 2 3 0 1 3 0 1 2 0 89 34 116 167 0 89 182 129 167 0 29 34 129 159 0 </values>  </instantiation>");
		map.put("Taillard-js-015-15-0.xml",
				"<instantiation id='sol1' type='solution' cost='1319'>  <list> s[][] e[] </list>  <values> 0 94 160 170 223 422 437 502 584 595 622 955 1047 1143 1223 218 292 344 544 610 667 745 753 760 851 930 948 999 1032 1131 0 203 382 520 633 683 793 814 821 875 943 1025 1045 1084 1159 323 437 483 552 582 635 735 793 886 918 1009 1039 1095 1131 1310 0 78 101 122 182 218 247 342 441 520 596 913 955 1072 1199 0 93 160 248 318 334 441 506 607 689 716 948 1101 1163 1177 0 18 117 137 141 484 552 584 690 820 943 986 1035 1072 1202 0 234 286 295 370 431 622 763 825 831 942 1119 1199 1202 1259 0 85 122 232 323 438 525 607 690 886 942 1027 1035 1159 1295 94 115 174 362 422 463 480 546 638 722 963 1032 1077 1159 1165 0 90 117 118 126 248 328 463 512 544 635 825 920 926 1164 0 49 126 431 582 635 638 722 756 784 844 918 1015 1082 1165 29 141 239 342 362 417 450 483 560 610 715 813 903 1035 1227 0 28 90 141 217 234 323 382 606 833 903 930 999 1069 1213 182 285 328 450 484 525 667 735 808 813 821 833 986 1163 1183 1306 1164 1227 1319 1295 1207 1268 1266 1310 1178 1237 1252 1271 1248 1280 </values>  </instantiation>");
		map.put("SocialGolfers-4-3-4-cp.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 0 0 0 1 1 1 2 2 2 3 3 3 0 2 3 0 1 3 0 1 2 1 2 3 3 2 1 0 2 3 2 1 0 3 1 0 3 2 1 2 3 0 0 2 1 1 0 3 </values>  </instantiation>");
		map.put("SportsScheduling-08.xml",
				"<instantiation id='sol1' type='solution'>  <list> h[][] a[][] m[][] hd[] ad[] </list>  <values> 0 0 1 2 3 3 5 2 1 2 3 4 0 0 4 3 0 1 0 1 2 6 4 5 0 1 2 1 1 2 4 7 6 4 6 3 5 6 5 7 6 7 5 7 3 6 5 7 4 7 6 7 4 2 5 3 0 1 9 17 20 18 25 13 10 16 19 24 5 6 22 21 2 11 4 12 14 27 23 26 3 7 15 8 5 1 2 0 7 4 6 3 </values>  </instantiation>");
		map.put("StillLife-03-06.xml",
				"<instantiation id='sol3' type='solution' cost='10'>  <list> x[][] s[][] </list>  <values> 1 1 0 1 1 0 1 1 0 1 0 1 0 0 0 0 1 1 3 3 4 2 3 2 3 3 4 3 6 3 2 2 2 2 3 2 </values>  </instantiation>");
		map.put("StillLife-wastage-03.xml",
				"<instantiation id='sol3' type='solution' cost='6'>  <list> x[][] w[][] ws[] z </list>  <values> 0 0 0 0 0 0 0 1 1 0 0 1 0 1 0 0 1 1 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 0 0 0 2 0 0 0 0 0 0 1 0 0 0 1 0 1 2 4 5 6 6 </values>  </instantiation>");
		map.put("StripPacking-C1P1.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] y[] w[] h[] r[] </list>  <values> 0 0 4 14 17 7 14 17 0 14 17 0 17 0 12 12 18 5 12 0 7 0 6 0 0 18 16 16 12 12 11 0 12 12 8 3 3 5 3 3 7 6 3 4 3 4 2 2 2 7 6 6 5 5 12 7 5 2 2 2 4 4 9 11 1 1 0 0 0 0 0 0 1 1 0 0 0 0 1 1 </values>  </instantiation>");
		map.put("Subisomorphism-A-10.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[] </list>  <values> 53 96 169 117 26 136 187 67 93 24 124 158 107 198 18 174 80 191 88 29 129 143 125 7 49 139 16 36 179 114 20 150 177 190 9 98 195 154 64 46 104 14 135 121 151 119 160 76 197 54 146 34 110 97 189 147 122 134 11 83 30 171 172 113 163 86 116 55 168 192 118 157 194 73 25 32 60 3 72 132 91 69 138 148 74 152 186 95 156 115 52 8 56 39 71 130 50 170 87 173 180 102 77 27 131 145 176 94 21 161 105 33 19 133 44 193 99 12 40 13 59 166 81 126 22 149 63 65 15 159 51 41 28 127 23 162 100 141 42 123 89 175 196 153 103 4 108 183 128 10 57 84 62 185 178 164 90 1 58 66 101 2 184 38 0 35 106 31 68 17 70 111 181 82 6 155 5 199 120 144 </values>  </instantiation>");
		map.put("Sudoku-s01a-alldiff.xml",
				"<instantiation id='sol1' type='solution'>  <list> x[][] </list>  <values> 8 4 5 6 3 2 1 7 9 7 3 2 9 1 8 6 5 4 1 9 6 7 4 5 3 2 8 6 8 3 5 7 4 9 1 2 4 5 7 2 9 1 8 3 6 2 1 9 8 6 3 5 4 7 3 6 1 4 2 9 7 8 5 5 7 4 1 8 6 2 9 3 9 2 8 3 5 7 4 6 1 </values>  </instantiation>");
		// map.put("Sudoku-s01a-table.xml",
		// "<instantiation id='sol1' type='solution'> <list> x[][] </list> <values> 8 4 5 6 3 2 1 7 9 7 3 2 9 1 8 6 5 4 1 9 6 7 4 5 3 2 8 6
		// 8 3 5 7 4 9 1 2 4 5
		// 7 2 9 1 8 3 6 2 1 9 8 6 3 5 4 7 3 6 1 4 2 9 7 8 5 5 7 4 1 8 6 2 9 3 9 2 8 3 5 7 4 6 1 </values> </instantiation>");
		map.put("Tpp-3-3-20-1.xml",
				"<instantiation id='sol2' type='solution' cost='126'>  <list> s[] d[] l[] c[] </list>  <values> 3 0 1 4 7 2 6 8 5 6 8 5 9 18 3 0 13 11 7 4 1 4 1 0 0 4 0 3 2 1 3 0 2 1 1 0 2 5 4 5 1 4 1 3 6 1 1 1 3 6 2 1 2 5 1 1 2 3 </values>  </instantiation>");
		map.put("TravellingSalesman-20-30-00.xml",
				"<instantiation id='sol1' type='solution' cost='176'>  <list> c[] d[] </list>  <values> 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 7 8 2 7 11 6 6 12 10 9 10 5 17 1 2 4 4 15 12 28 </values>  </instantiation>");
		map.put("Vrp-A-n32-k5.xml",
				"<instantiation id='sol1' type='solution' cost='3744'>  <list> x[][] u[] </list>  <values> 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 * 19 21 6 19 7 12 16 6 16 8 14 21 16 3 22 18 19 1 24 8 12 4 8 24 24 2 20 15 2 14 9 </values>  </instantiation>");
		map.put("Vrp-P-n16-k8.xml",
				"<instantiation id='sol24' type='solution' cost='464'>  <list> x[][] u[] </list>  <values> 0 1 1 1 1 0 1 0 1 1 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 * 19 30 16 23 30 31 23 28 8 27 34 30 34 19 34 </values>  </instantiation>"
						+ "<instantiation id='sol27' type='solution' cost='453'>  <list> x[][] u[] </list>  <values> 0 0 1 1 0 0 1 0 1 0 0 1 1 1 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 * 35 30 16 30 30 31 29 28 14 33 7 14 6 19 25 </values>  </instantiation>");
		map.put("Warehouse-opl.xml",
				"<instantiation id='sol4' type='solution' cost='383'>  <list> n s[] c[] o[] </list>  <values> 4 4 1 4 0 4 1 1 2 1 2 30 27 70 2 4 22 5 13 35 55 1 1 1 0 1 </values>  </instantiation>");
		return map.entrySet().stream().map(e -> new Object[] { e.getKey(), e.getValue() }).collect(Collectors.toList());
	}

	@Parameter(value = 0)
	public String fileName;

	@Parameter(value = 1)
	public String solution;

	@Test
	public void test() throws Exception {
		System.out.println("file=" + fileName);
		SolutionChecker sc = new SolutionChecker(true, path + fileName + ".lzma", new ByteArrayInputStream(solution.getBytes()));
		assertEquals(sc.violatedCtrs.size(), 0);
		assertEquals(sc.invalidObjs.size(), 0);
	}
}
